"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _reactMotion = require("react-motion");

var _glamor = require("glamor");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TextLoop = function (_React$PureComponent) {
    _inherits(TextLoop, _React$PureComponent);

    function TextLoop(props) {
        _classCallCheck(this, TextLoop);

        var _this = _possibleConstructorReturn(this, (TextLoop.__proto__ || Object.getPrototypeOf(TextLoop)).call(this, props));

        _initialiseProps.call(_this);

        _this.state = {
            currentWord: 0,
            wordCount: 0,
            width: props.initialWidth,
            height: props.initialHeight
        };
        return _this;
    }

    _createClass(TextLoop, [{
        key: "componentDidMount",
        value: function componentDidMount() {
            this.setDefaultWidth();
            this.tickInterval = setInterval(this.tick, this.props.speed);
        }
    }, {
        key: "componentWillUnmount",
        value: function componentWillUnmount() {
            clearInterval(this.tickInterval);
        }
    }, {
        key: "setDefaultWidth",
        value: function setDefaultWidth() {
            var _getDimensions = this.getDimensions(),
                height = _getDimensions.height,
                width = _getDimensions.width;

            this.setState(function () {
                return {
                    width: width,
                    height: height
                };
            });
        }
    }, {
        key: "getDimensions",
        value: function getDimensions() {
            if (this.wordBox == null) {
                return this.state;
            }

            return this.wordBox.getBoundingClientRect();
        }
    }, {
        key: "getStyles",
        value: function getStyles() {
            var _getDimensions2 = this.getDimensions(),
                height = _getDimensions2.height;

            return (0, _glamor.css)(this.props.style, {
                display: "inline-block",
                position: "relative",
                verticalAlign: "top",
                height: height
            });
        }
    }, {
        key: "getTextStyles",
        value: function getTextStyles() {
            return (0, _glamor.css)({
                whiteSpace: "nowrap",
                display: "inline-block",
                position: "absolute",
                left: 0,
                top: 0
            });
        }
    }, {
        key: "getTransitionMotionStyles",
        value: function getTransitionMotionStyles() {
            var _props = this.props,
                children = _props.children,
                springConfig = _props.springConfig;
            var _state = this.state,
                wordCount = _state.wordCount,
                currentWord = _state.currentWord;

            var options = _react2.default.Children.toArray(children);

            return [{
                key: "step" + wordCount,
                data: {
                    text: options[currentWord]
                },
                style: {
                    opacity: (0, _reactMotion.spring)(1, springConfig),
                    translate: (0, _reactMotion.spring)(0, springConfig)
                }
            }];
        }
    }, {
        key: "render",
        value: function render() {
            var _this2 = this;

            // Show first child while it's loading
            if (this.state.width === 0) {
                var children = _react2.default.Children.toArray(this.props.children)[0];
                return _react2.default.createElement(
                    "span",
                    { ref: function ref(n) {
                            _this2.wordBox = n;
                        } },
                    children
                );
            }

            return _react2.default.createElement(
                "div",
                this.getStyles(),
                _react2.default.createElement(
                    _reactMotion.TransitionMotion,
                    {
                        willLeave: this.willLeave,
                        willEnter: this.willEnter,
                        styles: this.getTransitionMotionStyles()
                    },
                    function (interpolatedStyles) {
                        var _getDimensions3 = _this2.getDimensions(),
                            height = _getDimensions3.height,
                            width = _getDimensions3.width;

                        return _react2.default.createElement(
                            "div",
                            {
                                style: {
                                    transition: "width " + _this2.props.adjustingSpeed + "ms linear",
                                    height: height,
                                    width: width
                                }
                            },
                            interpolatedStyles.map(function (config) {
                                return _react2.default.createElement(
                                    "div",
                                    _extends({}, _this2.getTextStyles(), {
                                        ref: function ref(n) {
                                            _this2.wordBox = n;
                                        },
                                        key: config.key,
                                        style: {
                                            opacity: config.style.opacity,
                                            transform: "translateY(" + config.style.translate + "px)"
                                        }
                                    }),
                                    config.data.text
                                );
                            })
                        );
                    }
                )
            );
        }
    }]);

    return TextLoop;
}(_react2.default.PureComponent);

var _initialiseProps = function _initialiseProps() {
    var _this3 = this;

    this.willLeave = function () {
        var _getDimensions4 = _this3.getDimensions(),
            height = _getDimensions4.height;

        return {
            opacity: (0, _reactMotion.spring)(0),
            translate: (0, _reactMotion.spring)(-height)
        };
    };

    this.willEnter = function () {
        var _getDimensions5 = _this3.getDimensions(),
            height = _getDimensions5.height;

        return {
            opacity: 0,
            translate: height
        };
    };

    this.tick = function () {
        _this3.setState(function (state, props) {
            return {
                currentWord: (state.currentWord + 1) % _react2.default.Children.count(props.children),
                wordCount: (state.wordCount + 1) % 1000 };
        });
    };
};

TextLoop.propTypes = {
    speed: _react2.default.PropTypes.number.isRequired,
    adjustingSpeed: _react2.default.PropTypes.number.isRequired,
    initialWidth: _react2.default.PropTypes.number.isRequired,
    initialHeight: _react2.default.PropTypes.number.isRequired,
    style: _react2.default.PropTypes.object,
    springConfig: _react2.default.PropTypes.object.isRequired,
    children: _react2.default.PropTypes.node.isRequired
};

TextLoop.defaultProps = {
    speed: 3000,
    adjustingSpeed: 150,
    springConfig: { stiffness: 340, damping: 30 },
    initialWidth: 0,
    initialHeight: 0
};

exports.default = TextLoop;